/**
 *
 */
package de.intevation.gnv.action;

import java.util.Collection;
import java.util.Iterator;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.w3c.dom.Node;

import de.intevation.gnv.action.sessionmodel.DiagrammOptions;
import de.intevation.gnv.action.sessionmodel.SessionModel;
import de.intevation.gnv.action.sessionmodel.SessionModelFactory;
import de.intevation.gnv.artifactdatabase.client.ArtifactDatabaseClient;
import de.intevation.gnv.artifactdatabase.client.ArtifactDatabaseClientFactory;
import de.intevation.gnv.artifactdatabase.objects.ArtifactDescription;
import de.intevation.gnv.artifactdatabase.objects.OutputMode;
import de.intevation.gnv.artifactdatabase.objects.OutputParameter;
import de.intevation.gnv.util.XMLUtils;
import de.intevation.gnv.util.XSLTransformer;

/**
 * @author Tim Englich <tim.englich@intevation.de>
 * 
 */
public class ChangeOptionsAction extends ArtifactDatabaseActionBase {
    /**
     * the logger, used to log exceptions and additonaly information
     */
    private static Logger log = Logger.getLogger(ChangeOptionsAction.class);

    /**
     * Constructor
     */
    public ChangeOptionsAction() {
        super();
    }

    @Override
    public ActionForward execute(ActionMapping mapping, ActionForm form,
                                 HttpServletRequest request,
                                 HttpServletResponse response) throws Exception {
        log.debug("NextArtifactStepAction.execute");
        try {
            SessionModel sm = SessionModelFactory.getInstance()
                    .getSessionModel(request);

            String target = request.getParameter("target");
            OutputMode outputMode = sm.getOutputMode(target);
            if (outputMode != null) {

                Collection<OutputParameter> op = outputMode
                        .getOutputParameters();
                if (op != null) {
                    Iterator<OutputParameter> it = op.iterator();
                    DiagrammOptions diagrammOptions = new DiagrammOptions();
                    while (it.hasNext()) {
                        OutputParameter parameter = it.next();
                        String value = request.getParameter(parameter.getName());
                        if (parameter.getType().equalsIgnoreCase("boolean")){
                            if (value == null){
                                value = "false";
                            }
                            else {
                                value = "true";
                            }
                        }
                        diagrammOptions.setValue(parameter.getName(),value);

                    }
                    sm.setDiagrammOptions(diagrammOptions);
                }
            }

            ArtifactDescription ad = sm.getArtifactDescription();
            ArtifactDatabaseClient adc = ArtifactDatabaseClientFactory
                                                .getInstance()
                                                .getArtifactDatabaseClient(getLocale(request));

            ArtifactDescription artifactDescription = adc
                    .getCurrentStepDescription(sm.getSelectedArtifactFactory(),
                                               sm.getCurrentArtifact(),
                                               true);

            Node currentUI = artifactDescription.getCurrentUI();
            XSLTransformer transformer = new XSLTransformer();

            String url = response.encodeURL(
                mapping.findForward("back").getPath());
            transformer.addParameter("back-url", url);

            String fisUrl = response.encodeURL(
                mapping.findForward("selectfis").getPath());
            transformer.addParameter("selectfis", fisUrl);

            String ui = transformer.transform(
                new XMLUtils().getNodeXPath(currentUI, "art:dynamic"),
                "UTF-8",
                request.getRealPath(XSL_SHEET_DYNAMIC
            ));

            if (ui != null && ui.length() > 1)
                request.setAttribute("ui", ui);

            String staticUI = transformer.transform(
                new XMLUtils().getNodeXPath(currentUI, "art:static"),
                "UTF-8",
                request.getRealPath(XSL_SHEET_STATIC));
            request.setAttribute("staticui", staticUI);

            Map tmpOuts = ad.getOutputModes();
            if (tmpOuts == null || tmpOuts.isEmpty()) {
                request.setAttribute("furthertargets", true);
            }
            else {
                request.setAttribute("diagramm", true);
            }

            return super.execute(mapping, form, request, response);
        } catch (Exception e) {
            log.error(e, e);
            request.setAttribute(
                    CommunicationKeys.REQUEST_EXCEPTION_MESSAGE_ID, e
                            .getMessage());
            return super.getExceptionForward(mapping);
        }
    }

}
